/**
 * EasyBeans
 * Copyright (C) 2006-2008 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: LogHandler.java 2546 2008-02-29 13:49:31Z sauthieg $
 * --------------------------------------------------------------------------
 */

package org.ow2.easybeans.examples.cxf;

import java.io.PrintStream;
import java.util.Set;

import javax.xml.namespace.QName;
import javax.xml.soap.SOAPMessage;
import javax.xml.ws.handler.MessageContext;
import javax.xml.ws.handler.soap.SOAPHandler;
import javax.xml.ws.handler.soap.SOAPMessageContext;

/**
 * Logging Handler, print the input and output SOAP Message.
 * @author Guillaume Sauthier
 */
public class LogHandler implements SOAPHandler<SOAPMessageContext> {

    /**
     * Change this to redirect output if desired.
     */
    private static PrintStream out = System.out;

    /**
     * Gets the header blocks that can be processed by this Handler instance.
     * @return Set of QNames of header blocks processed by this handler
     *         instance. QName is the qualified name of the outermost element of
     *         the Header block.
     */
    public Set<QName> getHeaders() {
        return null;
    }

    /**
     * The handleMessage method is invoked for normal processing of inbound and
     * outbound messages. Refer to the description of the handler framework in
     * the JAX-WS specification for full details.
     * @param context the message context.
     * @return always true
     */
    public boolean handleMessage(final SOAPMessageContext context) {
        logToSystemOut(context);
        return true;
    }

    /**
     * The handleFault method is invoked for fault message processing. Refer to
     * the description of the handler framework in the JAX-WS specification for
     * full details.
     * @param context the message context
     * @return always true
     */
    public boolean handleFault(final SOAPMessageContext context) {
        logToSystemOut(context);
        return true;
    }

    /**
     * Nothing to clean up.
     * @param messageContext the given context
     */
    public void close(final MessageContext messageContext) {
    }

    /**
     * Log to the given output the soap message context.
     * @param smc the given context to print.
     */
    private void logToSystemOut(final SOAPMessageContext smc) {
        Boolean outboundProperty = (Boolean) smc.get(MessageContext.MESSAGE_OUTBOUND_PROPERTY);

        if (outboundProperty.booleanValue()) {
            out.println("\nOutbound message:");
        } else {
            out.println("\nInbound message:");
        }

        SOAPMessage message = smc.getMessage();
        try {
            message.writeTo(out);
            out.println(); // just to add a newline
        } catch (Exception e) {
            out.println("Exception in handler: " + e);
        }
    }
}
